@@ -1,7 +1,7 @@
 /*
  * ebiin.c
  *
- * Copyright (c) Chris Putnam 2004-2013
+ * Copyright (c) Chris Putnam 2004-2018
  *
  * Program and source code released under the GPL version 2
  *
@@ -9,12 +9,18 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include "is_ws.h"
-#include "newstr.h"
-#include "newstr_conv.h"
+#include "str.h"
+#include "str_conv.h"
 #include "fields.h"
+#include "bu_auth.h"
+#include "marc_auth.h"
 #include "xml.h"
 #include "xml_encoding.h"
-#include "ebiin.h"
+#include "bibformats.h"
+
+static int ebiin_readf( FILE *fp, char *buf, int bufsize, int *bufpos, str *line, str *reference, int *fcharset );
+static int ebiin_processf( fields *ebiin, char *data, char *filename, long nref, param *p );
+
 
 /*****************************************************
  PUBLIC: void ebiin_initparams()
@@ -42,8 +48,8 @@ ebiin_initparams( param *p, const char *progname )
 	p->all      = NULL;
 	p->nall     = 0;
 
-	list_init( &(p->asis) );
-	list_init( &(p->corps) );
+	slist_init( &(p->asis) );
+	slist_init( &(p->corps) );
 
 	if ( !progname ) p->progname = NULL;
 	else p->progname = strdup( progname );
@@ -52,34 +58,34 @@ ebiin_initparams( param *p, const char *progname )
 /*****************************************************
  PUBLIC: int ebiin_readf()
 *****************************************************/
-int
-ebiin_readf( FILE *fp, char *buf, int bufsize, int *bufpos, newstr *line, newstr *reference, int *fcharset )
+static int
+ebiin_readf( FILE *fp, char *buf, int bufsize, int *bufpos, str *line, str *reference, int *fcharset )
 {
-	newstr tmp;
-	char *startptr = NULL, *endptr;
 	int haveref = 0, inref = 0, file_charset = CHARSET_UNKNOWN, m;
-	newstr_init( &tmp );
-	while ( !haveref && newstr_fget( fp, buf, bufsize, bufpos, line ) ) {
+	char *startptr = NULL, *endptr;
+	str tmp;
+	str_init( &tmp );
+	while ( !haveref && str_fget( fp, buf, bufsize, bufpos, line ) ) {
 		if ( line->data ) {
 			m = xml_getencoding( line );
 			if ( m!=CHARSET_UNKNOWN ) file_charset = m;
 		}
 		if ( line->data )
-			startptr = xml_findstart( line->data, "Publication" );
+			startptr = xml_find_start( line->data, "Publication" );
 		if ( startptr || inref ) {
-			if ( inref ) newstr_strcat( &tmp, line->data );
+			if ( inref ) str_strcat( &tmp, line );
 			else {
-				newstr_strcat( &tmp, startptr );
+				str_strcatc( &tmp, startptr );
 				inref = 1;
 			}
-			endptr = xml_findend( tmp.data, "Publication" );
+			endptr = xml_find_end( str_cstr( &tmp ), "Publication" );
 			if ( endptr ) {
-				newstr_segcpy( reference, tmp.data, endptr );
+				str_segcpy( reference, str_cstr( &tmp ), endptr );
 				haveref = 1;
 			}
 		}
 	}
-	newstr_free( &tmp );
+	str_free( &tmp );
 	*fcharset = file_charset;
 	return haveref;
 }
@@ -101,19 +107,19 @@ ebiin_doconvert( xml *node, fields *info, xml_convert *c, int nc, int *found )
 	int i, status;
 	char *d;
 
-	if ( !xml_hasdata( node ) ) goto out;
+	if ( !xml_has_value( node ) ) goto out;
 
-	d = xml_data( node );
+	d = xml_value_cstr( node );
 	for ( i=0; i<nc; ++i ) {
 		if ( c[i].a==NULL ) {
-			if ( xml_tagexact( node, c[i].in ) ) {
+			if ( xml_tag_matches( node, c[i].in ) ) {
 				*found = 1;
 				status = fields_add( info, c[i].out, d, c[i].level );
 				if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
 				else return BIBL_OK;
 			}
 		} else {
-			if ( xml_tag_attrib( node, c[i].in, c[i].a, c[i].aval)){
+			if ( xml_tag_has_attribute( node, c[i].in, c[i].a, c[i].aval ) ){
 				*found = 1;
 				status = fields_add( info, c[i].out, d, c[i].level );
 				if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
@@ -135,8 +141,8 @@ static int
 ebiin_title( xml *node, fields *info, int title_level )
 {
 	int status;
-	if ( xml_hasdata( node ) ) {
-		status = fields_add( info, "TITLE", xml_data( node ), title_level );
+	if ( xml_has_value( node ) ) {
+		status = fields_add( info, "TITLE", xml_value_cstr( node ), title_level );
 		if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
 	}
 	return BIBL_OK;
@@ -148,54 +154,88 @@ ebiin_title( xml *node, fields *info, int title_level )
  *             <MedlineDate>2003 Jan-Feb</MedlineDate>
  */
 static int
-ebiin_medlinedate_year( fields *info, char *p, newstr *s, int level, char **end )
+ebiin_medlinedate_year( fields *info, char *p, int level, char **end )
 {
-	int status;
-	*end = newstr_cpytodelim( s, p, " \t\n\r", 0 );
-	if ( newstr_memerr( s ) ) return BIBL_ERR_MEMERR;
-	if ( s->len ) {
-		status = fields_add( info, "PARTYEAR", s->data, level );
-		if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
+	int fstatus, status = BIBL_OK;
+	str s;
+
+	str_init( &s );
+
+	*end = str_cpytodelim( &s, p, " \t\n\r", 0 );
+	if ( str_memerr( &s ) ) {
+		status = BIBL_ERR_MEMERR;
+		goto out;
 	}
-	return BIBL_OK;
+	if ( str_has_value( &s ) ) {
+		fstatus = fields_add( info, "PARTDATE:YEAR", str_cstr( &s ), level );
+		if ( fstatus!=FIELDS_OK ) status = BIBL_ERR_MEMERR;
+	}
+out:
+	str_free( &s );
+	return status;
 }
 static int
-ebiin_medlinedate_month( fields *info, char *p, newstr *s, int level, char **end )
+ebiin_medlinedate_month( fields *info, char *p, int level, char **end )
 {
-	int status;
-	*end = newstr_cpytodelim( s, p, " \t\n\r", 0 );
-	newstr_findreplace( s, "-", "/" );
-	if ( newstr_memerr( s ) ) return BIBL_ERR_MEMERR;
-	if ( s->len ) {
-		status = fields_add( info, "PARTMONTH", s->data, level );
-		if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
+	int fstatus, status = BIBL_OK;
+	str s;
+
+	str_init( &s );
+
+	*end = str_cpytodelim( &s, p, " \t\n\r", 0 );
+	str_findreplace( &s, "-", "/" );
+	if ( str_memerr( &s ) ) {
+		status = BIBL_ERR_MEMERR;
+		goto out;
 	}
-	return BIBL_OK;
-}
-static int
-ebiin_medlinedate_day( fields *info, char *p, newstr *s, int level, char **end )
-{
-	int status;
-	*end = newstr_cpytodelim( s, p, " \t\n\r", 0 );
-	if ( newstr_memerr( s ) ) return BIBL_ERR_MEMERR;
-	if ( s->len ) {
-		status = fields_add( info, "PARTDAY", s->data, level );
-		if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
+	if ( str_has_value( &s ) ) {
+		fstatus = fields_add( info, "PARTDATE:MONTH", str_cstr( &s ), level );
+		if ( fstatus!=FIELDS_OK ) status = BIBL_ERR_MEMERR;
 	}
-	return BIBL_OK;
+out:
+	str_free( &s );
+	return status;
 }
+
 static int
-ebiin_medlinedate( fields *info, char *p, int level )
+ebiin_medlinedate_day( fields *info, char *p, int level, char **end )
 {
-	int status;
-	newstr s;
-	newstr_init( &s );
-	status = ebiin_medlinedate_year( info, skip_ws( p ), &s, level, &p );
-	if ( status==BIBL_OK && *p )
-		status = ebiin_medlinedate_month( info, skip_ws( p ), &s, level, &p );
-	if ( status==BIBL_OK && *p )
-		status = ebiin_medlinedate_day( info, skip_ws( p ), &s, level, &p );
-	newstr_free( &s );
+	int fstatus, status = BIBL_OK;
+	str s;
+
+	str_init( &s );
+
+	*end = str_cpytodelim( &s, p, " \t\n\r", 0 );
+	if ( str_memerr( &s ) ) {
+		status = BIBL_ERR_MEMERR;
+		goto out;
+	}
+	if ( str_has_value( &s ) ) {
+		fstatus = fields_add( info, "PARTDATE:DAY", str_cstr( &s ), level );
+		if ( fstatus!=FIELDS_OK ) status = BIBL_ERR_MEMERR;
+	}
+out:
+	str_free( &s );
+	return status;
+}
+
+static int
+ebiin_medlinedate( fields *info, xml *node, int level )
+{
+	int status = BIBL_OK;
+	char *p;
+
+	if ( !xml_has_value( node ) ) return status;
+
+	p = xml_value_cstr( node );
+
+	if ( *p )
+		status = ebiin_medlinedate_year( info, skip_ws( p ), level, &p );
+	if ( *p && status==BIBL_OK )
+		status = ebiin_medlinedate_month( info, skip_ws( p ), level, &p );
+	if ( *p && status==BIBL_OK )
+		status = ebiin_medlinedate_day( info, skip_ws( p ), level, &p );
+
 	return status;
 }
 
@@ -231,21 +271,21 @@ static int
 ebiin_journal1( xml *node, fields *info )
 {
 	xml_convert c[] = {
-		{ "ISSN",     NULL, NULL, "ISSN",      1 },
-		{ "Volume",   NULL, NULL, "VOLUME",    1 },
-		{ "Issue",    NULL, NULL, "ISSUE",     1 },
-		{ "Year",     NULL, NULL, "PARTYEAR",  1 },
-		{ "Month",    NULL, NULL, "PARTMONTH", 1 },
-		{ "Day",      NULL, NULL, "PARTDAY",   1 },
-		{ "Language", NULL, NULL, "LANGUAGE",  1 },
+		{ "ISSN",     NULL, NULL, "ISSN",           1 },
+		{ "Volume",   NULL, NULL, "VOLUME",         1 },
+		{ "Issue",    NULL, NULL, "ISSUE",          1 },
+		{ "Year",     NULL, NULL, "PARTDATE:YEAR",  1 },
+		{ "Month",    NULL, NULL, "PARTDATE:MONTH", 1 },
+		{ "Day",      NULL, NULL, "PARTDATE:DAY",   1 },
+		{ "Language", NULL, NULL, "LANGUAGE",       1 },
 	};
 	int nc = sizeof( c ) / sizeof( c[0] ), status, found;
-	if ( xml_hasdata( node ) ) {
+	if ( xml_has_value( node ) ) {
 		status = ebiin_doconvert( node, info, c, nc, &found );
 		if ( status!=BIBL_OK ) return status;
 		if ( !found ) {
-			if ( xml_tagexact( node, "MedlineDate" ) ) {
-				status = ebiin_medlinedate( info, xml_data( node ), 1 );
+			if ( xml_tag_matches( node, "MedlineDate" ) ) {
+				status = ebiin_medlinedate( info, node, LEVEL_HOST );
 				if ( status!=BIBL_OK ) return status;
 			}
 		}
@@ -270,26 +310,26 @@ ebiin_pages( fields *info, char *p )
 {
 	int i, status, ret = BIBL_OK;
 	const int level = 1;
-	newstr sp, ep, *up;
+	str sp, ep, *up;
 
-	newstrs_init( &sp, &ep, NULL );
+	strs_init( &sp, &ep, NULL );
 
 	/* ...start page */
-	p = newstr_cpytodelim( &sp, skip_ws( p ), "-", 1 );
-	if ( newstr_memerr( &sp ) ) {
+	p = str_cpytodelim( &sp, skip_ws( p ), "-", 1 );
+	if ( str_memerr( &sp ) ) {
 		ret = BIBL_ERR_MEMERR;
 		goto out;
 	}
 
 	/* ...end page */
-	p = newstr_cpytodelim( &ep, skip_ws( p ), " \t\n\r", 0 );
-	if ( newstr_memerr( &ep ) ) {
+	(void) str_cpytodelim( &ep, skip_ws( p ), " \t\n\r", 0 );
+	if ( str_memerr( &ep ) ) {
 		ret = BIBL_ERR_MEMERR;
 		goto out;
 	}
 
 	if ( sp.len ) {
-		status = fields_add( info, "PAGESTART", sp.data, level );
+		status = fields_add( info, "PAGES:START", sp.data, level );
 		if ( status!=FIELDS_OK ) {
 			ret = BIBL_ERR_MEMERR;
 			goto out;
@@ -299,22 +339,22 @@ ebiin_pages( fields *info, char *p )
 		if ( sp.len > ep.len ) {
 			for ( i=sp.len-ep.len; i<sp.len; ++i )
 				sp.data[i] = ep.data[i-sp.len+ep.len];
-				up = &(sp);
+			up = &(sp);
 		} else up = &(ep);
-		status = fields_add( info, "PAGEEND", up->data, level );
+		status = fields_add( info, "PAGES:STOP", up->data, level );
 		if ( status!=FIELDS_OK ) ret = BIBL_ERR_MEMERR;
 	}
 
 out:
-	newstrs_free( &sp, &ep, NULL );
+	strs_free( &sp, &ep, NULL );
 	return ret;
 }
 static int
 ebiin_pagination( xml *node, fields *info )
 {
 	int status;
-	if ( xml_tagexact( node, "Pages" ) && node->value ) {
-		status = ebiin_pages( info, xml_data( node ) );
+	if ( xml_tag_matches_has_value( node, "Pages" ) ) {
+		status = ebiin_pages( info, xml_value_cstr( node ) );
 		if ( status!=BIBL_OK ) return status;
 	}
 	if ( node->down ) {
@@ -336,8 +376,8 @@ static int
 ebiin_abstract( xml *node, fields *info )
 {
 	int status;
-	if ( xml_hasdata( node ) && xml_tagexact( node, "AbstractText" ) ) {
-		status = fields_add( info, "ABSTRACT", xml_data( node ), 0 );
+	if ( xml_tag_matches_has_value( node, "AbstractText" ) ) {
+		status = fields_add( info, "ABSTRACT", xml_value_cstr( node ), 0 );
 		if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
 	}
 	else if ( node->next ) {
@@ -357,32 +397,32 @@ ebiin_abstract( xml *node, fields *info )
  * </AuthorList>
  */
 static int
-ebiin_author( xml *node, newstr *name )
+ebiin_author( xml *node, str *name )
 {
 	int status;
 	char *p;
-	if ( xml_tagexact( node, "LastName" ) ) {
+	if ( xml_tag_matches( node, "LastName" ) ) {
 		if ( name->len ) {
-			newstr_prepend( name, "|" );
-			newstr_prepend( name, xml_data( node ) );
+			str_prepend( name, "|" );
+			str_prepend( name, xml_value_cstr( node ) );
 		}
-		else newstr_strcat( name, xml_data( node ) );
-	} else if ( xml_tagexact( node, "ForeName" ) || 
-	            xml_tagexact( node, "FirstName" ) ) {
-		p = xml_data( node );
+		else str_strcat( name, xml_value( node ) );
+	} else if ( xml_tag_matches( node, "ForeName" ) ||
+	            xml_tag_matches( node, "FirstName" ) ) {
+		p = xml_value_cstr( node );
 		while ( p && *p ) {
-			if ( name->len ) newstr_addchar( name, '|' );
-			while ( *p && *p==' ' ) p++;
-			while ( *p && *p!=' ' ) newstr_addchar( name, *p++ );
+			if ( name->len ) str_addchar( name, '|' );
+			while ( *p==' ' ) p++;
+			while ( *p && *p!=' ' ) str_addchar( name, *p++ );
 		}
-	} else if ( xml_tagexact( node, "Initials" ) && !strchr( name->data, '|' ) ) {
-		p = xml_data( node );
+	} else if ( xml_tag_matches( node, "Initials" ) && !strchr( name->data, '|' ) ) {
+		p = xml_value_cstr( node );
 		while ( p && *p ) {
-			if ( name->len ) newstr_addchar( name, '|' );
-			if ( !is_ws(*p ) ) newstr_addchar( name, *p++ );
+			if ( name->len ) str_addchar( name, '|' );
+			if ( !is_ws(*p ) ) str_addchar( name, *p++ );
 		}
 	}
-	if ( newstr_memerr( name ) ) return BIBL_ERR_MEMERR;
+	if ( str_memerr( name ) ) return BIBL_ERR_MEMERR;
 		 
 	if ( node->down ) {
 		status = ebiin_author( node->down, name );
@@ -399,24 +439,24 @@ static int
 ebiin_authorlist( xml *node, fields *info, int level )
 {
 	int fstatus, status = BIBL_OK;
-	newstr name;
+	str name;
 
-	newstr_init( &name );
+	str_init( &name );
 	node = node->down;
 	while ( node ) {
-		if ( xml_tagexact( node, "Author" ) && node->down ) {
+		if ( xml_tag_matches( node, "Author" ) && node->down ) {
 			status = ebiin_author( node->down, &name );
 			if ( status!=BIBL_OK ) goto out;
 			if ( name.len ) {
 				fstatus = fields_add(info,"AUTHOR",name.data,level);
 				if ( fstatus!=FIELDS_OK ) { status = BIBL_ERR_MEMERR; goto out; }
-				newstr_empty( &name );
+				str_empty( &name );
 			}
 		}
 		node = node->next;
 	}
 out:
-	newstr_free( &name );
+	str_free( &name );
 	return status;
 }
 
@@ -436,8 +476,8 @@ static int
 ebiin_journal2( xml *node, fields *info )
 {
 	int status;
-	if ( xml_tagwithdata( node, "TitleAbbreviation" ) ) {
-		status = fields_add( info, "TITLE", xml_data( node ), 1 );
+	if ( xml_tag_matches_has_value( node, "TitleAbbreviation" ) ) {
+		status = fields_add( info, "TITLE", xml_value_cstr( node ), 1 );
 		if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
 	}
 	if ( node->down ) {
@@ -465,8 +505,8 @@ static int
 ebiin_meshheading( xml *node, fields *info )
 {
 	int status;
-	if ( xml_tagwithdata( node, "DescriptorName" ) ) {
-		status = fields_add( info, "KEYWORD", xml_data( node ), 0 );
+	if ( xml_tag_matches_has_value( node, "DescriptorName" ) ) {
+		status = fields_add( info, "KEYWORD", xml_value_cstr( node ), 0 );
 		if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
 	}
 	if ( node->next ) {
@@ -480,7 +520,7 @@ static int
 ebiin_meshheadinglist( xml *node, fields *info )
 {
 	int status;
-	if ( xml_tagexact( node, "MeshHeading" ) && node->down ) {
+	if ( xml_tag_matches( node, "MeshHeading" ) && node->down ) {
 		status = ebiin_meshheading( node->down, info );
 		if ( status!=BIBL_OK ) return status;
 	}
@@ -495,63 +535,72 @@ static int
 ebiin_book( xml *node, fields *info, int book_level )
 {
 	xml_convert book[] = {
-		{ "Publisher",              NULL, NULL, "PUBLISHER",  0 },
-		{ "Language",               NULL, NULL, "LANGUAGE",   0 },
-		{ "ISBN10",                 NULL, NULL, "ISBN",       0 },
-		{ "ISBN13",                 NULL, NULL, "ISBN13",     0 },
-		{ "Year",                   NULL, NULL, "YEAR",       0 },
-		{ "Month",                  NULL, NULL, "MONTH",      0 },
-		{ "Day",                    NULL, NULL, "DAY",        0 },
-		{ "PageTotal",              NULL, NULL, "TOTALPAGES", 0 },
-		{ "SeriesName",             NULL, NULL, "TITLE",      1 },
-		{ "SeriesISSN",             NULL, NULL, "ISSN",       0 },
-		{ "OtherReportInformation", NULL, NULL, "NOTES",      0 },
-		{ "Edition",                NULL, NULL, "EDITION",    0 },
+		{ "Publisher",              NULL, NULL, "PUBLISHER",      0 },
+		{ "Language",               NULL, NULL, "LANGUAGE",       0 },
+		{ "ISBN10",                 NULL, NULL, "ISBN",           0 },
+		{ "ISBN13",                 NULL, NULL, "ISBN13",         0 },
+		{ "Year",                   NULL, NULL, "DATE:YEAR",      0 },
+		{ "Month",                  NULL, NULL, "DATE:MONTH",     0 },
+		{ "Day",                    NULL, NULL, "DATE:DAY",       0 },
+		{ "PageTotal",              NULL, NULL, "PAGES:TOTAL",    0 },
+		{ "SeriesName",             NULL, NULL, "TITLE",          1 },
+		{ "SeriesISSN",             NULL, NULL, "ISSN",           0 },
+		{ "OtherReportInformation", NULL, NULL, "NOTES",          0 },
+		{ "Edition",                NULL, NULL, "EDITION",        0 },
 	};
 	int nbook = sizeof( book ) / sizeof( book[0] );
 	xml_convert inbook[] = {
-		{ "Publisher",              NULL, NULL, "PUBLISHER",  1 },
-		{ "Language",               NULL, NULL, "LANGUAGE",   0 },
-		{ "ISBN10",                 NULL, NULL, "ISBN",       1 },
-		{ "ISBN13",                 NULL, NULL, "ISBN13",     1 },
-		{ "Year",                   NULL, NULL, "PARTYEAR",   1 },
-		{ "Month",                  NULL, NULL, "PARTMONTH",  1 },
-		{ "Day",                    NULL, NULL, "PARTDAY",    1 },
-		{ "PageTotal",              NULL, NULL, "TOTALPAGES", 1 },
-		{ "SeriesName",             NULL, NULL, "TITLE",      2 },
-		{ "SeriesISSN",             NULL, NULL, "ISSN",       1 },
-		{ "OtherReportInformation", NULL, NULL, "NOTES",      1 },
-		{ "Edition",                NULL, NULL, "EDITION",    1 },
+		{ "Publisher",              NULL, NULL, "PUBLISHER",      1 },
+		{ "Language",               NULL, NULL, "LANGUAGE",       0 },
+		{ "ISBN10",                 NULL, NULL, "ISBN",           1 },
+		{ "ISBN13",                 NULL, NULL, "ISBN13",         1 },
+		{ "Year",                   NULL, NULL, "PARTDATE:YEAR",  1 },
+		{ "Month",                  NULL, NULL, "PARTDATE:MONTH", 1 },
+		{ "Day",                    NULL, NULL, "PARTDATE:DAY",   1 },
+		{ "PageTotal",              NULL, NULL, "PAGES:TOTAL",    1 },
+		{ "SeriesName",             NULL, NULL, "TITLE",          2 },
+		{ "SeriesISSN",             NULL, NULL, "ISSN",           1 },
+		{ "OtherReportInformation", NULL, NULL, "NOTES",          1 },
+		{ "Edition",                NULL, NULL, "EDITION",        1 },
 	};
 	int ninbook = sizeof( inbook ) / sizeof( inbook[0] );
-	xml_convert *c;
 	int nc, status, found;
-	if ( book_level==0 ) { c = book; nc = nbook; }
-	else { c = inbook; nc = ninbook; }
-	if ( xml_hasdata( node ) ) {
-		status = ebiin_doconvert( node, info, c, nc, &found );
-		if ( status!=BIBL_OK ) return status;
-		if ( !found ) {
-			status = BIBL_OK;
-			if ( xml_tagexact( node, "MedlineDate" ) )
-				status = ebiin_medlinedate( info, xml_data( node ), book_level);
-			else if ( xml_tagexact( node, "Title" ) )
-				status = ebiin_title( node, info, book_level );
-			else if ( xml_tagexact( node, "Pagination" ) && node->down )
-				status = ebiin_pagination( node->down, info );
-			else if ( xml_tagexact( node, "Abstract" ) && node->down )
-				status = ebiin_abstract( node->down, info );
-			else if ( xml_tagexact( node, "AuthorList" ) )
-				status = ebiin_authorlist( node, info, book_level );
-			else if ( xml_tagexact( node, "PubDate" ) && node->down)
-				status = ebiin_book( node->down, info, book_level );
-			if ( status!=BIBL_OK ) return status;
-		}
+	xml_convert *c;
+
+	if ( book_level==0 ) {
+		c  = book;
+		nc = nbook;
 	}
+	else {
+		c  = inbook;
+		nc = ninbook;
+	}
+
+	status = ebiin_doconvert( node, info, c, nc, &found );
+	if ( status!=BIBL_OK ) return status;
+
+	if ( !found ) {
+		status = BIBL_OK;
+		if ( xml_tag_matches( node, "MedlineDate" ) )
+			status = ebiin_medlinedate( info, node, book_level );
+		else if ( xml_tag_matches( node, "Title" ) )
+			status = ebiin_title( node, info, book_level );
+		else if ( xml_tag_matches( node, "Pagination" ) && node->down )
+			status = ebiin_pagination( node->down, info );
+		else if ( xml_tag_matches( node, "Abstract" ) && node->down )
+			status = ebiin_abstract( node->down, info );
+		else if ( xml_tag_matches( node, "AuthorList" ) )
+			status = ebiin_authorlist( node, info, book_level );
+		else if ( xml_tag_matches( node, "PubDate" ) && node->down )
+			status = ebiin_book( node->down, info, book_level );
+		if ( status!=BIBL_OK ) return status;
+	}
+
 	if ( node->next ) {
 		status = ebiin_book( node->next, info, book_level );
 		if ( status!=BIBL_OK ) return status;
 	}
+
 	return BIBL_OK;
 }
 
@@ -560,18 +609,18 @@ ebiin_article( xml *node, fields *info )
 {
 	int status = BIBL_OK;
 
-	if ( xml_tagexact( node, "Journal" ) ) 
+	if ( xml_tag_matches( node, "Journal" ) )
 		status = ebiin_journal1( node, info );
-	else if ( node->down && ( xml_tagexact( node, "Book" ) || 
-			xml_tagexact(node, "Report") )) 
+	else if ( node->down && ( xml_tag_matches( node, "Book" ) ||
+			xml_tag_matches(node, "Report") ))
 		status = ebiin_book( node->down, info, 1 );
-	else if ( xml_tagexact( node, "ArticleTitle" ) )
+	else if ( xml_tag_matches( node, "ArticleTitle" ) )
 		status = ebiin_title( node, info, 0 );
-	else if ( xml_tagexact( node, "Pagination" ) && node->down )
+	else if ( xml_tag_matches( node, "Pagination" ) && node->down )
 		status = ebiin_pagination( node->down, info );
-	else if ( xml_tagexact( node, "Abstract" ) && node->down )
+	else if ( xml_tag_matches( node, "Abstract" ) && node->down )
 		status = ebiin_abstract( node->down, info );
-	else if ( xml_tagexact( node, "AuthorList" ) )
+	else if ( xml_tag_matches( node, "AuthorList" ) )
 		status = ebiin_authorlist( node, info, 0 );
 	if ( status!=BIBL_OK ) return status;
 
@@ -586,17 +635,17 @@ ebiin_article( xml *node, fields *info )
 static int
 ebiin_publication( xml *node, fields *info )
 {
-	int status;
+	int status = BIBL_OK;
 	if ( node->down ) {
-		if ( xml_tagexact( node, "Article" ) )
+		if ( xml_tag_matches( node, "Article" ) )
 			status = ebiin_article( node->down, info );
-		else if ( xml_tagexact( node, "Book" ) )
+		else if ( xml_tag_matches( node, "Book" ) )
 			status = ebiin_book( node->down, info, 0 );
-		else if ( xml_tagexact( node, "Report" ) )
+		else if ( xml_tag_matches( node, "Report" ) )
 			status = ebiin_book( node->down, info, 0 );
-		else if ( xml_tagexact( node, "JournalInfo" ) )
+		else if ( xml_tag_matches( node, "JournalInfo" ) )
 			status = ebiin_journal2( node->down, info );
-		else if ( xml_tagexact( node, "MeshHeadingList" ) )
+		else if ( xml_tag_matches( node, "MeshHeadingList" ) )
 			status = ebiin_meshheadinglist( node->down, info );
 		if ( status!=BIBL_OK ) return status;
 	}
@@ -612,11 +661,11 @@ static int
 ebiin_fixtype( xml *node, fields *info )
 {
 	char *resource = NULL, *issuance = NULL, *genre1 = NULL, *genre2 = NULL;
-	newstr *type;
+	str *type;
 	int reslvl, isslvl, gen1lvl, gen2lvl;
 	int status;
 
-	type = xml_getattrib( node, "Type" );
+	type = xml_attribute( node, "Type" );
 	if ( !type || type->len==0 ) return BIBL_OK;
 
 	if ( !strcmp( type->data, "JournalArticle" ) ) {
@@ -653,11 +702,21 @@ ebiin_fixtype( xml *node, fields *info )
 		if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
 	}
 	if ( genre1 ) {
-		status = fields_add( info, "GENRE", genre1, gen1lvl );
+		if ( is_marc_genre( genre1 ) )
+			status = fields_add( info, "GENRE:MARC", genre1, gen1lvl );
+		else if ( is_bu_genre( genre1 ) )
+			status = fields_add( info, "GENRE:BIBUTILS", genre1, gen1lvl );
+		else
+			status = fields_add( info, "GENRE:UNKNOWN", genre1, gen1lvl );
 		if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
 	}
 	if ( genre2 ) {
-		status = fields_add( info, "GENRE", genre2, gen2lvl );
+		if ( is_marc_genre( genre2 ) )
+			status = fields_add( info, "GENRE:MARC", genre2, gen2lvl );
+		else if ( is_bu_genre( genre2 ) )
+			status = fields_add( info, "GENRE:BIBUTILS", genre2, gen2lvl );
+		else
+			status = fields_add( info, "GENRE:UNKNOWN", genre2, gen2lvl );
 		if ( status!=FIELDS_OK ) return BIBL_ERR_MEMERR;
 	}
 
@@ -668,7 +727,7 @@ static int
 ebiin_assembleref( xml *node, fields *info )
 {
 	int status;
-	if ( xml_tagexact( node, "Publication" ) && node->down ) {
+	if ( xml_tag_matches( node, "Publication" ) && node->down ) {
 		status = ebiin_fixtype( node, info );
 		if ( status!=BIBL_OK ) return status;
 		status = ebiin_publication( node->down, info );
@@ -684,14 +743,14 @@ ebiin_assembleref( xml *node, fields *info )
 	return BIBL_OK;
 }
 
-int
-ebiin_processf( fields *ebiin, char *data, char *filename, long nref )
+static int
+ebiin_processf( fields *ebiin, char *data, char *filename, long nref, param *p )
 {
 	int status;
 	xml top;
 
 	xml_init( &top );
-	xml_tree( data, &top );
+	xml_parse( data, &top );
 	status = ebiin_assembleref( &top, ebiin );
 	xml_free( &top );
 
